import unittest
import os, sys, shutil

test_root = os.path.abspath(os.path.dirname(__file__))

import breezedb

database = os.path.join(test_root, 'db_temp')
table = 'users'

class TestField(unittest.TestCase):

    def test_field_exists_true(self):
        # Check for a field that does exist
        result = breezedb.field_exists('id', table, database)
        self.assertEqual(True, result)

    def test_field_exists_false(self):
        # Check for a field that does not exist
        result = breezedb.field_exists('test', table, database)
        self.assertEqual(False, result)

    def test_get_fieldtype(self):
        # Return the field type of 'id' (int)
        expected = 'int'
        result = breezedb.get_field_type('id', table, database)
        self.assertEqual(expected, result)

    def test_get_fieldtype_inexistent(self):
        # Attemp to get the field type of an inexistent field
        with self.assertRaises(breezedb.BreezeException):
            breezedb.get_field_type('test', table, database)

    def test_create_field(self):
        # Add a new field to the table
        breezedb.create_field('new_field', 'string', table, database)

    def test_create_field_existing(self):
        # Attempt to add an already existing field
        with self.assertRaises(breezedb.BreezeException):
            breezedb.create_field('new_field', 'string', table, database) 

    def test_remove_field(self):
        # Remove the previously created field
        breezedb.remove_field('new_field', table, database)

    def test_remove_field_inexistent(self):
        # Attempt to remove the field again
        with self.assertRaises(breezedb.BreezeException):
            breezedb.remove_field('new_field', table, database)

    def test_empty_field(self):
        # Empty the contents of the 'name' field
        breezedb.empty_field('name', table, database)

    def test_empty_field_inexistent(self):
        # Attempt to empty the contents of an inexistent field
        with self.assertRaises(breezedb.BreezeException):
            breezedb.empty_field('none', table, database)

    def test_get_element_list(self):
        # Get an element list from the 'last_name' field
        expected = ['McPerson1', 'McPerson2', 'McPerson3', 'McPerson4',
                    'McPerson5']
        result = breezedb.get_element_list('last_name', table, database)
        self.assertEqual(expected, result)

    def test_get_element_list_inexistent(self):
        # Attempt to get an element list from an inexistent field
        with self.assertRaises(breezedb.BreezeException):
            breezedb.get_element_list('test_field', table, database)

if __name__ == "__main__":
    # Remove previous temp copy
    if os.path.isdir(os.path.join(test_root, 'db_temp')):
        shutil.rmtree(os.path.join(test_root, 'db_temp'))
    # Create temp copy of the database
    shutil.copytree(os.path.join(test_root, 'db'),
        os.path.join(test_root, 'db_temp'))
    # Begin tests
    unittest.main()




import unittest
import os, sys, shutil

test_root = os.path.abspath(os.path.dirname(__file__))

import breezedb

database = os.path.join(test_root, 'db_temp')
table = 'cities'
field = 'name'

class TestElement(unittest.TestCase):

    def test_element_exists_true(self):
        # Check for an element that does exist
        result = breezedb.element_exists(2, field, table, database)
        self.assertEqual(True, result)

    def test_element_exists_false(self):
        # Check for an element that does not exist
        result = breezedb.element_exists(10, field, table, database)
        self.assertEqual(False, result)
        
    def test_get_element_content(self):
        # Get the data of element located in position 1
        expected = 'Paris'
        result = breezedb.get_element_content(1, field, table, database)
        self.assertEqual(expected, result)

    def test_get_element_content_inexistent(self):
        # Attempt to get the data of an inexistent element
        with self.assertRaises(breezedb.BreezeException):
            breezedb.get_element_content(10, field, table, database)

    def test_find_element(self):
        # Find the index for 'Madrid'
        expected = 3
        result = breezedb.find_element('Madrid', field, table, database)
        # Must treat the result as integer
        self.assertEqual(expected, int(result[0]))

    def test_find_element_inexistent(self):
        # Attempt to find 'Tokyo'
        expected = []
        result = breezedb.find_element('Tokyo', field, table, database)
        # Must return an empty array
        self.assertEqual(expected, result)

    def test_modify_element(self):
        # Modify the last element
        breezedb.modify_element(4, 'Nowhere', field, table, database)

    def test_modify_element_inexistent(self):
        # Modify the last element
        with self.assertRaises(breezedb.BreezeException):
            breezedb.modify_element(35, 'Test Name', field, table, database)

    def test_empty_element(self):
        # Empty the content of element 0
        breezedb.empty_element(0, field, table, database)

    def test_empty_element_inexistent(self):
        # Attempt to empty the content of an inexistent element
        with self.assertRaises(breezedb.BreezeException):
            breezedb.empty_element(10, field, table, database)

    def test_remove_row(self):
        # Remove the complete row in position 2
        breezedb.remove_element_row(2, table, database)

    def test_remove_row_inexistent(self):
        # Attemptto remove an inexistent row
        with self.assertRaises(breezedb.BreezeException):
            breezedb.remove_element_row(100, table, database)

if __name__ == "__main__":
    # Remove previous temp copy
    if os.path.isdir(os.path.join(test_root, 'db_temp')):
        shutil.rmtree(os.path.join(test_root, 'db_temp'))
    # Create temp copy of the database
    shutil.copytree(os.path.join(test_root, 'db'),
        os.path.join(test_root, 'db_temp'))
    # Begin tests
    unittest.main()




import unittest
import os, sys, shutil

test_root = os.path.abspath(os.path.dirname(__file__))

import breezedb

class TestDBOperations(unittest.TestCase):

    def test_create_breezedb(self):
        # Create a new database
        path = test_root
        name = 'test_database1'
        breezedb.create_db(path, name)

    def test_create_breezedb_existing(self):
        # Try to create the database again
        with self.assertRaises(breezedb.BreezeException):
            path = test_root
            name = 'test_database1'
            breezedb.create_db(path, name)

    def test_remove_breezedb(self):
        # Remove previously created database
        path = os.path.join(test_root,'test_database1')
        breezedb.remove_db(path)
        # Remove temp database
        temp = os.path.join(test_root,'db_temp')
        breezedb.remove_db(temp)

    def test_remove_breezedb_inexistent(self):
        # Try to remove the databases again
        with self.assertRaises(breezedb.BreezeException):
            path = 'test_database1'
            breezedb.remove_db(path)
            temp = 'db_temp'
            breezedb.remove_db(temp)

if __name__ == "__main__":
    # Remove previous temp copy
    if os.path.isdir(os.path.join(test_root, 'db_temp')):
        shutil.rmtree(os.path.join(test_root, 'db_temp'))
    # Create temp copy of the database
    shutil.copytree(os.path.join(test_root, 'db'),
        os.path.join(test_root, 'db_temp'))
    # Begin tests
    unittest.main()




# -*- coding: utf-8 -*-
#
# This file is part of breezedb - https://github.com/RMed/breezedb_python
#
# Copyright (C) 2013  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
.. module:: field
    :platform: Unix, Windows
    :synopsis: Field related operations.

.. moduleauthor:: Rafael Medina García <rafamedgar@gmail.com>
"""

import xml.etree.ElementTree as XML
import os
from breezedb.breeze_exceptions import BreezeException

def field_exists(field_name, table_name, database):
    """ Check whether a field exists in the table or not.

        :param str field_name: name of the field to check
        :param str table_name: name of the table that contains the field
        :param str database: path to the database

        :returns: True or False
    """    
    table_file = os.path.join(database, table_name, 'tableinfo.breeze')
    table_tree = XML.parse(table_file)
    table_root = table_tree.getroot()

    exists = False
    for field in table_root:
        if field.text == field_name:
            exists = True
            break

    if not exists:
        return False

    field_path = os.path.join(database, table_name, field_name)
    if os.path.isfile(field_path):
        return True
    else:
        return False

def get_field_type(field_name, table_name, database):
    """ Get the data type of a field from the <type> tag.

        :param str field_name: name of the field to get the type from
        :param str table_name: name of the table that contains the field
        :param str database: path to the database

        :returns str: type of the field

        :raises BreezeException: field does not exist
    """
    if not field_exists(field_name, table_name, database):
        raise BreezeException('field', 'field does not exist')

    field_file = os.path.join(database, table_name, field_name)
    field_tree = XML.parse(field_file)
    field_root = field_tree.getroot()

    return field_root.find('type').text

def create_field(field_name, field_type, table_name, database):
    """ Create a new field in the table.

        Add a new <field> element to the tableinfo.breeze file and create the
        corresponding field structure.

        :param str field_name: name for the new field
        :param str field_type: type of the new field
        :param str table_name: name of the table that will contain the field
        :param str database: path to the database

        :raises BreezeException: database is not writable,
            a field `field_name` already exists
    """
    can_write = os.access(database, os.W_OK)
    if not can_write:
        raise BreezeException('field', 'cannot write to database')

    if field_exists(field_name, table_name, database):
        raise BreezeException('field', 'field already exists in the table')

    try:
        field_file = os.path.join(database, table_name, field_name)

        breeze_tag = XML.Element('breeze')
        field_tree = XML.ElementTree(breeze_tag)
        field_root = field_tree.getroot()

        new_type = XML.Element('type')
        new_type.text = field_type

        field_root.append(new_type)
        field_tree.write(field_file)

        table_file = os.path.join(database, table_name, 'tableinfo.breeze')
        table_tree = XML.parse(table_file)
        table_root = table_tree.getroot()

        new_field = XML.Element('field')
        new_field.text = field_name
        table_root.append(new_field)

        table_tree.write(table_file)

    except IOError:
        raise BreezeException('field', 'error writing to file')

def rename_field(field_name, table_name, database, new_name):
    """ Rename a field.

        :param str field_name: current name of the field
        :param str table_name: name of the table that contains the field
        :param str database: path to the database
        :param str new_name: new name for the field

        :raises BreezeException: database is not writable,
            the field to rename does not exist,
            a field `new_name` already exists
    """
    can_write = os.access(database, os.W_OK)
    if not can_write:
        raise BreezeException('field', 'cannot write to database')

    if not field_exists(field_name, table_name, database):
        raise BreezeException('field', 'field does not exist in the table')

    if field_exists(new_name, table_name, database):
        raise BreezeException('field', 'the field %s already exists', new_name)

    try:
        table_file = os.path.join(database, table_name, 'tableinfo.breeze')
        table_tree = XML.parse(table_file)
        table_root = table_tree.getroot()

        for field in table_root:
            if field.text == field_name:
                field.text = new_name
                break

        table_tree.write(table_file)

        src = os.path.join(database, table_name, field_name)
        dst = os.path.join(database, table_name, new_name)
        os.rename(src, dst)

    except IOError:
        raise BreezeException('field', 'could not rename the field')

    except OSError:
        raise BreezeException('field', 'could not rename the file')

def remove_field(field_name, table_name, database):
    """ Remove a field from the table.

        Remove the corresponding file and <field> element
        from the tableinfo.breeze file

        :param str field_name: name of the field to remove
        :param str table_name: name of the table that contains the field
        :param str database: path to the database
        
        :raises BreezeException: database is not writable,
            field does not exist
    """
    can_write = os.access(database, os.W_OK)
    if not can_write:
        raise BreezeException('field', 'cannot write to database')

    if not field_exists(field_name, table_name, database):
        raise BreezeException('field', 'field does not exist in the table')

    try:
        table_file = os.path.join(database, table_name, 'tableinfo.breeze')
        table_tree = XML.parse(table_file)
        table_root = table_tree.getroot()

        for field in table_root:
            if field.text == field_name:
                table_root.remove(field)
                break

        table_tree.write(table_file)

        field_file = os.path.join (database, table_name, field_name)
        os.remove(field_file)

    except OSError:
        raise BreezeException('field', 'could not remove file')

def empty_field(field_name, table_name, database):
    """ Empty the contents of a field.

        Remove the all the <element> tags from the file but
        preserve the <type> tab.

        :param str field_name: name of the field to empty
        :param str table_name: name of the table that contains the field
        :param str database: path to the database

        :raises BreezeException: database is not writable,
            field does not exist
    """
    can_write = os.access(database, os.W_OK)
    if not can_write:
        raise BreezeException('field', 'cannot write to database')

    if not field_exists(field_name, table_name, database):
        raise BreezeException('field', 'field does not exist in the table')

    field_file = os.path.join(database, table_name, field_name)
    field_tree = XML.parse(field_file)
    field_root = field_tree.getroot()

    for element in field_root.iter('element'):
        field_root.remove(element)

    field_tree.write(field_file)

def get_element_list(field_name, table_name, database):
    """ Get a list of elements contained in the field.

        :param str field_name: name of the field to get the elements from
        :param str table_name: name of the table that contains the field
        :param str database: path to the database

        :returns: list of elements

        :raises BreezeException: field does not exist
    """
    elemlist = []

    if not field_exists(field_name, table_name, database):
        raise BreezeException('field', 'field does not exist in the table')

    field_file = os.path.join(database, table_name, field_name)
    field_tree = XML.parse(field_file)
    field_root = field_tree.getroot()

    for element in field_root.iter('element'):
        elemlist.append(element.text)

    return elemlist




# -*- coding: utf-8 -*-
#
# This file is part of breezedb - https://github.com/RMed/breezedb_python
#
# Copyright (C) 2013  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
.. module:: element
    :platform: Unix, Windows
    :synopsis: Element related operations.

.. moduleauthor:: Rafael Medina García <rafamedgar@gmail.com>
"""

import xml.etree.ElementTree as XML
import os, string
import breezedb.field as field
from breezedb.breeze_exceptions import BreezeException

def element_exists(element_index, field_name, table_name, database):
    """ Check whether an element exist in the field.

        :param int element_index: index of the element to check
        :param str field_name: name of the field that contains the element
        :param str table_name: name of the table that contains the field
        :param str database: path to the database

        :returns: True or false

        :raises BreezeException: supplied index is not a positive integer
    """
    if element_index < 0:
        raise BreezeException('element', 'index needs to be a positive integer')

    field_file = os.path.join(database, table_name, field_name)
    field_tree = XML.parse(field_file)
    field_root = field_tree.getroot()

    last_index = int(field_root[-1].get('index'))
    if element_index <= last_index:
        return True
    else:
        return False

def parse_element_content(field_type, content):
    """ Parse the content of the element depending on the type.

        :param str field_type: type of the field that contains the element
        :param str content: content to parse

        :returns: parsed content according to the type (String, Int,
            Boolean, etc)
    """
    if field_type == 'string':
        return str(content)
    elif field_type == 'int':
        return int(content)
    elif field_type == 'float' or field_type == 'double':
        # Python's float type has double precision
        return float(content)
    elif field_type == 'boolean':
        # boolean type is represented by a 0 (false) or a 1 (true)
        if content == '0':
            return False
        else:
            return True

def get_element_content(element_index, field_name, table_name, database,
    parse = True):
    """ Get the data contained in an element.

        :param int element_index: index of the element
        :param str field_name: name of the field that contains the element
        :param str table_name: name of the table that contains the field
        :param str database: path to the database
        :param Boolean parse: whether to parse the content according to
            the type of the field or not

        :returns str: content of the element
    """
    if not element_exists(element_index, field_name, table_name, database):
        raise BreezeException('element', 'the element does not exist')

    field_file = os.path.join(database, table_name, field_name)
    field_tree = XML.parse(field_file)
    field_root = field_tree.getroot()

    if parse:
        field_type = field.get_field_type(field_name, table_name, database)
        content = field_root.findall('element')[element_index].text
        return parse_element_content(field_type, content)     

    return field_root.findall('element')[element_index].text

def find_element(to_find, field_name, table_name, database,
        ignore_case = True):
    """ Find elements in the field.

        :param str to_find: data to find
        :param str field_name: name of the field that contains the element
        :param str table_name: name of the table that contains the field
        :param str database: path to the database
        :param Boolean ignore_case: whether or not to ignore the case
            when searching

        :returns: list of indexes that match the criteria
    """
    indexlist = []

    field_file = os.path.join(database, table_name, field_name)
    field_tree = XML.parse(field_file)
    field_root = field_tree.getroot()

    for element in field_root.iter('element'):
        if ignore_case:
            if string.lower(str(to_find)) in string.lower(str(element.text)):
                indexlist.append(element.get('index'))
        else:
            if str(to_find) in str(element.text):
                indexlist.append(element.get('index'))

    return indexlist

def modify_element(element_index, new_content, field_name,
                    table_name, database):
    """ Modify the content of an element.

        :param int element_index: index of the element to modify
        :param str new_content: new content to store in the element
        :param str field_name: name of the field that contains the element
        :param str table_name: name of the table that contains the field
        :param str database: path to the database

        :raises BreezeException: element does not exist
    """
    if not element_exists(element_index, field_name, table_name, database):
        raise BreezeException('element', 'the element does not exist')

    field_file = os.path.join(database, table_name, field_name)
    field_tree = XML.parse(field_file)
    field_root = field_tree.getroot()

    field_root.findall('element')[element_index].text = str(new_content)

    field_tree.write(field_file)

def empty_element(element_index, field_name, table_name, database):
    """ Empty the content of an element.

        The content is emptied instead of removing the whole element because
        a single element cannot be removed, as the indexes of other fields
        would not match.

        :param int element_index: index of the element to empty
        :param str field_name: name of the field that contains the element
        :param str table_name: name of the table that contains the field
        :param str database: path to the database

        :raises BreezeException: element does not exist
    """
    if not element_exists(element_index, field_name, table_name, database):
        raise BreezeException('element', 'the element does not exist')

    field_file = os.path.join(database, table_name, field_name)
    field_tree = XML.parse(field_file)
    field_root = field_tree.getroot()

    field_root.findall('element')[element_index].text = ""

    field_tree.write(field_file)

def remove_element_row(element_index, table_name, database):
    """ Remove an element row from the specified table.

        Removes all the elements that correspond to that index and then
        updates the rest of the indexes of the files.

        :param int element_index: index of the element row to remove
        :param str field_name: name of the field that contains the element
        :param str table_name: name of the table that contains the field
        :param str database: path to the database

        :raises BreezeException: element does not exist,
            element row cannot be removed
    """
    table_file = os.path.join(database, table_name, 'tableinfo.breeze')
    table_tree = XML.parse(table_file)
    table_root = table_tree.getroot()

    for field in table_root:

        if not element_exists(element_index, field.text, table_name, database):
            raise BreezeException('element', 'the element does not exist')

        try:
            field_file = os.path.join(database, table_name, field.text)
            field_tree = XML.parse(field_file)
            field_root = field_tree.getroot()        

            field_root.remove(field_root.findall('element')[element_index])

            for element in field_root.iter('element'):
                if int(element.get('index')) > element_index:
                    element.set('index', str(int(element.get('index')) - 1))

            field_tree.write(field_file)

        except OSError:
            raise BreezeException('element', 'could not remove element')




# -*- coding: utf-8 -*-
#
# This file is part of breezedb - https://github.com/RMed/breezedb_python
#
# Copyright (C) 2013  Rafael Medina García <rafamedgar@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from distutils.core import setup

setup(
    name = 'breezedb',
    version = '1.0.1',
    description = 'A simple file-based database engine',
    author = 'Rafael Medina García',
    author_email = 'rafamedgar@gmail.com',
    url = 'https://github.com/RMed/breezedb_python',
    license = 'LGPLv3',
    packages = ['breezedb'],
    package_dir = {'breezedb': 'breezedb'},

    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Database :: Database Engines/Servers',
        'Topic :: Software Development :: Libraries :: Python Modules'
        ]
    )
